data

package
v0.0.0-...-42b5f6e Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Dec 22, 2020 License: BSD-3-Clause Imports: 1 Imported by: 0

Documentation

Overview

Packag data contains various data interfaces and prototypical implementations.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func XYUVRange

func XYUVRange(xyuvs XYUVer) (xmin, xmax, ymin, ymax, umin, umax, vmin, vmax float64)

XYUVRange returns the minimum and maximum x, y, u and v values.

Types

type Boxplots

type Boxplots []struct {
	X                        float64
	Min, Q1, Median, Q3, Max float64
	Outlier                  []float64
}

Boxplots implememnts the Boxplotter interface.

func (Boxplots) Boxplot

func (b Boxplots) Boxplot(i int) (x, min, q1, median, q3, max float64, outlier []float64)

func (Boxplots) Len

func (b Boxplots) Len() int

type Boxplotter

type Boxplotter interface {
	// Len returns the number of boxes.
	Len() int

	// Boxplot returns an data for the i'th boxplot.
	Boxplot(i int) (x, min, q1, median, q3, max float64, outlier []float64)
}

Boxplotter wraps the Len and Boxplot methods.

type XYTexter

type XYTexter interface {
	// Len returns the number of data points.
	Len() int

	// XYText returns a coordinate (x, y) and a string.
	XYText(int) (x, y float64, t string)
}

XYText wraps the Len and XYText methods.

type XYTexts

type XYTexts []struct {
	X, Y float64
	Text string
}

XYTexts implements the XYTexter interface.

func (XYTexts) Len

func (d XYTexts) Len() int

func (XYTexts) XYText

func (d XYTexts) XYText(i int) (x, y float64, text string)

type XYUVer

type XYUVer interface {
	// Len returns the number of x, y, u, v quadruples.
	Len() int

	// XYUV returns an x, y, u, v quadruple.
	XYUV(int) (x, y, u, v float64)
}

XYUVer wraps the Len and XYUV methods.

type XYUVs

type XYUVs []struct{ X, Y, U, V float64 }

XYUVs implements the XYUVer interface.

func (XYUVs) Len

func (d XYUVs) Len() int

func (XYUVs) XYUV

func (d XYUVs) XYUV(i int) (x, y, u, v float64)

Jump to